"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
const meta_data_1 = require("./meta_data");
function _DeserializeMap(data, type, target, instantiationMethod) {
    if (typeof data !== "object") {
        throw new Error("Expected input to be of type `object` but received: " + typeof data);
    }
    if (target === null || target === void 0)
        target = {};
    if (data === null || data === void 0) {
        return null;
    }
    const keys = Object.keys(data);
    for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        const value = data[key];
        if (value !== void 0) {
            target[meta_data_1.MetaData.deserializeKeyTransform(key)] = _Deserialize(data[key], type, target[key], instantiationMethod);
        }
    }
    return target;
}
function DeserializeMap(data, type, target, instantiationMethod) {
    if (instantiationMethod === void 0) {
        instantiationMethod = meta_data_1.MetaData.deserializeInstantationMethod;
    }
    return _DeserializeMap(data, type, target, instantiationMethod);
}
exports.DeserializeMap = DeserializeMap;
function _DeserializeArray(data, type, target, instantiationMethod) {
    if (!Array.isArray(data)) {
        throw new Error("Expected input to be an array but received: " + typeof data);
    }
    if (!Array.isArray(target))
        target = [];
    target.length = data.length;
    for (let i = 0; i < data.length; i++) {
        target[i] = _Deserialize(data[i], type, target[i], instantiationMethod);
    }
    return target;
}
function DeserializeArray(data, type, target, instantiationMethod) {
    if (instantiationMethod === void 0) {
        instantiationMethod = meta_data_1.MetaData.deserializeInstantationMethod;
    }
    return _DeserializeArray(data, type, target, instantiationMethod);
}
exports.DeserializeArray = DeserializeArray;
function DeserializePrimitive(data, type, target) {
    if (type === Date) {
        const deserializedDate = new Date(data);
        if (target instanceof Date) {
            target.setTime(deserializedDate.getTime());
        }
        else {
            return deserializedDate;
        }
    }
    else if (type === RegExp) {
        const fragments = data.match(/\/(.*?)\/([gimy])?$/);
        return new RegExp(fragments[1], fragments[2] || "");
    }
    else if (data === null) {
        return null;
    }
    else {
        return type(data);
    }
}
function DeserializeJSON(data, transformKeys = true, target) {
    if (Array.isArray(data)) {
        if (!Array.isArray(target))
            target = new Array(data.length);
        target.length = data.length;
        for (let i = 0; i < data.length; i++) {
            target[i] = DeserializeJSON(data[i], transformKeys, target[i]);
        }
        return target;
    }
    const type = typeof data;
    if (type === "object") {
        const retn = (target && typeof target === "object" ? target : {});
        const keys = Object.keys(data);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            const value = data[key];
            if (value !== void 0) {
                const retnKey = transformKeys ? meta_data_1.MetaData.deserializeKeyTransform(key) : key;
                retn[retnKey] = DeserializeJSON(data[key], transformKeys);
            }
        }
        return retn;
    }
    else if (type === "function") {
        throw new Error("Cannot deserialize a function, input is not a valid json object");
    }
    return data;
}
exports.DeserializeJSON = DeserializeJSON;
function _Deserialize(data, type, target, instantiationMethod) {
    const metadataList = meta_data_1.MetaData.getMetaDataForType(type);
    if (metadataList === null) {
        if (typeof type === "function") {
            if (util_1.isPrimitiveType(type)) {
                return DeserializePrimitive(data, type, target);
            }
            switch (instantiationMethod) {
                case util_1.InstantiationMethod.New:
                    return new type();
                case util_1.InstantiationMethod.ObjectCreate:
                    return Object.create(type.prototype);
                default:
                    return {};
            }
        }
        return null;
    }
    target = util_1.getTarget(type, target, instantiationMethod);
    for (let i = 0; i < metadataList.length; i++) {
        const metadata = metadataList[i];
        if (metadata.deserializedKey === null)
            continue;
        const source = data[metadata.getDeserializedKey()];
        if (source === void 0)
            continue;
        const keyName = metadata.keyName;
        const flags = metadata.flags;
        if ((flags & 32) !== 0) {
            target[keyName] = _DeserializeMap(source, metadata.deserializedType, target[keyName], instantiationMethod);
        }
        else if ((flags & 8) !== 0) {
            target[keyName] = _DeserializeArray(source, metadata.deserializedType, target[keyName], instantiationMethod);
        }
        else if ((flags & 2) !== 0) {
            target[keyName] = DeserializePrimitive(source, metadata.deserializedType, target[keyName]);
        }
        else if ((flags & 8192) !== 0) {
            target[keyName] = _Deserialize(source, metadata.deserializedType, target[keyName], instantiationMethod);
        }
        else if ((flags & 128) !== 0) {
            target[keyName] = DeserializeJSON(source, (flags & 512) !== 0, instantiationMethod);
        }
        else if ((flags & 2048) !== 0) {
            target[keyName] = metadata.deserializedType(source, target[keyName], instantiationMethod);
        }
    }
    if (typeof type.onDeserialized === "function") {
        const value = type.onDeserialized(data, target, instantiationMethod);
        if (value !== void 0)
            return value;
    }
    return target;
}
function Deserialize(data, type, target, instantiationMethod) {
    if (instantiationMethod === void 0) {
        instantiationMethod = meta_data_1.MetaData.deserializeInstantationMethod;
    }
    return _Deserialize(data, type, target, instantiationMethod);
}
exports.Deserialize = Deserialize;
function DeserializeRaw(data, type, target) {
    return _Deserialize(data, type, target, util_1.InstantiationMethod.None);
}
exports.DeserializeRaw = DeserializeRaw;
function DeserializeArrayRaw(data, type, target) {
    return _DeserializeArray(data, type, target, util_1.InstantiationMethod.None);
}
exports.DeserializeArrayRaw = DeserializeArrayRaw;
function DeserializeMapRaw(data, type, target) {
    return _DeserializeMap(data, type, target, util_1.InstantiationMethod.None);
}
exports.DeserializeMapRaw = DeserializeMapRaw;
//# sourceMappingURL=deserialize.js.map